#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "src/engine/SCons/UtilTests.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import SCons.compat

import io
import os
import sys
import unittest
from collections import UserDict, UserList, UserString

import TestCmd

import SCons.Errors

from SCons.Util import *

try:
    eval('unicode')
except NameError:
    HasUnicode = False
else:
    HasUnicode = True


class OutBuffer(object):
    def __init__(self):
        self.buffer = ""

    def write(self, str):
        self.buffer = self.buffer + str


class dictifyTestCase(unittest.TestCase):
    def test_dictify(self):
        """Test the dictify() function"""
        r = SCons.Util.dictify(['a', 'b', 'c'], [1, 2, 3])
        assert r == {'a': 1, 'b': 2, 'c': 3}, r

        r = {}
        SCons.Util.dictify(['a'], [1], r)
        SCons.Util.dictify(['b'], [2], r)
        SCons.Util.dictify(['c'], [3], r)
        assert r == {'a': 1, 'b': 2, 'c': 3}, r


class UtilTestCase(unittest.TestCase):
    def test_splitext(self):
        assert splitext('foo') == ('foo', '')
        assert splitext('foo.bar') == ('foo', '.bar')
        assert splitext(os.path.join('foo.bar', 'blat')) == (os.path.join('foo.bar', 'blat'), '')

    class Node(object):
        def __init__(self, name, children=[]):
            self.children = children
            self.name = name
            self.nocache = None

        def __str__(self):
            return self.name

        def exists(self):
            return 1

        def rexists(self):
            return 1

        def has_builder(self):
            return 1

        def has_explicit_builder(self):
            return 1

        def side_effect(self):
            return 1

        def precious(self):
            return 1

        def always_build(self):
            return 1

        def is_up_to_date(self):
            return 1

        def noclean(self):
            return 1

    def tree_case_1(self):
        """Fixture for the render_tree() and print_tree() tests."""
        windows_h = self.Node("windows.h")
        stdlib_h = self.Node("stdlib.h")
        stdio_h = self.Node("stdio.h")
        bar_c = self.Node("bar.c", [stdlib_h, windows_h])
        bar_o = self.Node("bar.o", [bar_c])
        foo_c = self.Node("foo.c", [stdio_h])
        foo_o = self.Node("foo.o", [foo_c])
        foo = self.Node("foo", [foo_o, bar_o])

        expect = """\
+-foo
  +-foo.o
  | +-foo.c
  |   +-stdio.h
  +-bar.o
    +-bar.c
      +-stdlib.h
      +-windows.h
"""

        lines = expect.split('\n')[:-1]
        lines = ['[E BSPACN ]' + l for l in lines]
        withtags = '\n'.join(lines) + '\n'

        return foo, expect, withtags

    def tree_case_2(self, prune=1):
        """Fixture for the render_tree() and print_tree() tests."""

        types_h = self.Node('types.h')
        malloc_h = self.Node('malloc.h')
        stdlib_h = self.Node('stdlib.h', [types_h, malloc_h])
        bar_h = self.Node('bar.h', [stdlib_h])
        blat_h = self.Node('blat.h', [stdlib_h])
        blat_c = self.Node('blat.c', [blat_h, bar_h])
        blat_o = self.Node('blat.o', [blat_c])

        expect = """\
+-blat.o
  +-blat.c
    +-blat.h
    | +-stdlib.h
    |   +-types.h
    |   +-malloc.h
    +-bar.h
"""
        if prune:
            expect += """      +-[stdlib.h]
"""
        else:
            expect += """      +-stdlib.h
        +-types.h
        +-malloc.h
"""

        lines = expect.split('\n')[:-1]
        lines = ['[E BSPACN ]' + l for l in lines]
        withtags = '\n'.join(lines) + '\n'

        return blat_o, expect, withtags

    def test_render_tree(self):
        """Test the render_tree() function"""

        def get_children(node):
            return node.children

        node, expect, withtags = self.tree_case_1()
        actual = render_tree(node, get_children)
        assert expect == actual, (expect, actual)

        node, expect, withtags = self.tree_case_2()
        actual = render_tree(node, get_children, 1)
        assert expect == actual, (expect, actual)

        # Ensure that we can call render_tree on the same Node
        # again. This wasn't possible in version 2.4.1 and earlier
        # due to a bug in render_tree (visited was set to {} as default
        # parameter)
        actual = render_tree(node, get_children, 1)
        assert expect == actual, (expect, actual)

    def test_print_tree(self):
        """Test the print_tree() function"""

        def get_children(node):
            return node.children

        save_stdout = sys.stdout

        try:
            node, expect, withtags = self.tree_case_1()

            if sys.version_info.major < 3:
                IOStream = io.BytesIO
            else:
                IOStream = io.StringIO

            sys.stdout = IOStream()
            print_tree(node, get_children)
            actual = sys.stdout.getvalue()
            assert expect == actual, (expect, actual)

            sys.stdout = IOStream()
            print_tree(node, get_children, showtags=1)
            actual = sys.stdout.getvalue()
            assert withtags == actual, (withtags, actual)

            # Test that explicitly setting prune to zero works
            # the same as the default (see above)
            node, expect, withtags = self.tree_case_2(prune=0)

            sys.stdout = IOStream()
            print_tree(node, get_children, 0)
            actual = sys.stdout.getvalue()
            assert expect == actual, (expect, actual)

            sys.stdout = IOStream()
            print_tree(node, get_children, 0, showtags=1)
            actual = sys.stdout.getvalue()
            assert withtags == actual, (withtags, actual)

            # Test output with prune=1
            node, expect, withtags = self.tree_case_2(prune=1)

            sys.stdout = IOStream()
            print_tree(node, get_children, 1)
            actual = sys.stdout.getvalue()
            assert expect == actual, (expect, actual)

            # Ensure that we can call print_tree on the same Node
            # again. This wasn't possible in version 2.4.1 and earlier
            # due to a bug in print_tree (visited was set to {} as default
            # parameter)
            sys.stdout = IOStream()
            print_tree(node, get_children, 1)
            actual = sys.stdout.getvalue()
            assert expect == actual, (expect, actual)

            sys.stdout = IOStream()
            print_tree(node, get_children, 1, showtags=1)
            actual = sys.stdout.getvalue()
            assert withtags == actual, (withtags, actual)
        finally:
            sys.stdout = save_stdout

    def test_is_Dict(self):
        assert is_Dict({})
        assert is_Dict(UserDict())

        # os.environ is not a dictionary in python 3
        if sys.version_info < (3, 0):
            assert is_Dict(os.environ)

        try:
            class mydict(dict):
                pass
        except TypeError:
            pass
        else:
            assert is_Dict(mydict({}))
        assert not is_Dict([])
        assert not is_Dict(())
        assert not is_Dict("")
        if HasUnicode:
            exec ("assert not is_Dict(u'')")

    def test_is_List(self):
        assert is_List([])
        assert is_List(UserList())
        try:
            class mylist(list):
                pass
        except TypeError:
            pass
        else:
            assert is_List(mylist([]))
        assert not is_List(())
        assert not is_List({})
        assert not is_List("")
        if HasUnicode:
            exec ("assert not is_List(u'')")

    def test_is_String(self):
        assert is_String("")
        if HasUnicode:
            exec ("assert is_String(u'')")
        assert is_String(UserString(''))
        try:
            class mystr(str):
                pass
        except TypeError:
            pass
        else:
            assert is_String(mystr(''))
        assert not is_String({})
        assert not is_String([])
        assert not is_String(())

    def test_is_Tuple(self):
        assert is_Tuple(())
        try:
            class mytuple(tuple):
                pass
        except TypeError:
            pass
        else:
            assert is_Tuple(mytuple(()))
        assert not is_Tuple([])
        assert not is_Tuple({})
        assert not is_Tuple("")
        if HasUnicode:
            exec ("assert not is_Tuple(u'')")

    def test_to_Bytes(self):
        """ Test the to_Bytes method"""
        if not PY3:
            self.assertEqual(to_bytes(UnicodeType('Hello')),
                             bytearray(u'Hello', 'utf-8'),
                             "Check that to_bytes creates byte array when presented with unicode string. PY2 only")

    def test_to_String(self):
        """Test the to_String() method."""
        assert to_String(1) == "1", to_String(1)
        assert to_String([1, 2, 3]) == str([1, 2, 3]), to_String([1, 2, 3])
        assert to_String("foo") == "foo", to_String("foo")
        assert to_String(None) == 'None'
        # test low level string converters too
        assert to_str(None) == 'None'
        assert to_bytes(None) == b'None'

        s1 = UserString('blah')
        assert to_String(s1) == s1, s1
        assert to_String(s1) == 'blah', s1

        class Derived(UserString):
            pass

        s2 = Derived('foo')
        assert to_String(s2) == s2, s2
        assert to_String(s2) == 'foo', s2

        if HasUnicode:
            s3 = UserString(unicode('bar'))
            assert to_String(s3) == s3, s3
            assert to_String(s3) == unicode('bar'), s3
            assert isinstance(to_String(s3), unicode), \
                type(to_String(s3))

        if HasUnicode:
            s4 = unicode('baz')
            assert to_String(s4) == unicode('baz'), to_String(s4)
            assert isinstance(to_String(s4), unicode), \
                type(to_String(s4))

    def test_WhereIs(self):
        test = TestCmd.TestCmd(workdir='')

        sub1_xxx_exe = test.workpath('sub1', 'xxx.exe')
        sub2_xxx_exe = test.workpath('sub2', 'xxx.exe')
        sub3_xxx_exe = test.workpath('sub3', 'xxx.exe')
        sub4_xxx_exe = test.workpath('sub4', 'xxx.exe')

        test.subdir('subdir', 'sub1', 'sub2', 'sub3', 'sub4')

        if sys.platform != 'win32':
            test.write(sub1_xxx_exe, "\n")

        os.mkdir(sub2_xxx_exe)

        test.write(sub3_xxx_exe, "\n")
        os.chmod(sub3_xxx_exe, 0o777)

        test.write(sub4_xxx_exe, "\n")
        os.chmod(sub4_xxx_exe, 0o777)

        env_path = os.environ['PATH']

        try:
            pathdirs_1234 = [test.workpath('sub1'),
                             test.workpath('sub2'),
                             test.workpath('sub3'),
                             test.workpath('sub4'),
                             ] + env_path.split(os.pathsep)

            pathdirs_1243 = [test.workpath('sub1'),
                             test.workpath('sub2'),
                             test.workpath('sub4'),
                             test.workpath('sub3'),
                             ] + env_path.split(os.pathsep)

            os.environ['PATH'] = os.pathsep.join(pathdirs_1234)
            wi = WhereIs('xxx.exe')
            assert wi == test.workpath(sub3_xxx_exe), wi
            wi = WhereIs('xxx.exe', pathdirs_1243)
            assert wi == test.workpath(sub4_xxx_exe), wi
            wi = WhereIs('xxx.exe', os.pathsep.join(pathdirs_1243))
            assert wi == test.workpath(sub4_xxx_exe), wi

            wi = WhereIs('xxx.exe', reject=sub3_xxx_exe)
            assert wi == test.workpath(sub4_xxx_exe), wi
            wi = WhereIs('xxx.exe', pathdirs_1243, reject=sub3_xxx_exe)
            assert wi == test.workpath(sub4_xxx_exe), wi

            os.environ['PATH'] = os.pathsep.join(pathdirs_1243)
            wi = WhereIs('xxx.exe')
            assert wi == test.workpath(sub4_xxx_exe), wi
            wi = WhereIs('xxx.exe', pathdirs_1234)
            assert wi == test.workpath(sub3_xxx_exe), wi
            wi = WhereIs('xxx.exe', os.pathsep.join(pathdirs_1234))
            assert wi == test.workpath(sub3_xxx_exe), wi

            if sys.platform == 'win32':
                wi = WhereIs('xxx', pathext='')
                assert wi is None, wi

                wi = WhereIs('xxx', pathext='.exe')
                assert wi == test.workpath(sub4_xxx_exe), wi

                wi = WhereIs('xxx', path=pathdirs_1234, pathext='.BAT;.EXE')
                assert wi.lower() == test.workpath(sub3_xxx_exe).lower(), wi

                # Test that we return a normalized path even when
                # the path contains forward slashes.
                forward_slash = test.workpath('') + '/sub3'
                wi = WhereIs('xxx', path=forward_slash, pathext='.EXE')
                assert wi.lower() == test.workpath(sub3_xxx_exe).lower(), wi

            del os.environ['PATH']
            wi = WhereIs('xxx.exe')
            assert wi is None, wi

        finally:
            os.environ['PATH'] = env_path

    def test_get_env_var(self):
        """Testing get_environment_var()."""
        assert get_environment_var("$FOO") == "FOO", get_environment_var("$FOO")
        assert get_environment_var("${BAR}") == "BAR", get_environment_var("${BAR}")
        assert get_environment_var("$FOO_BAR1234") == "FOO_BAR1234", get_environment_var("$FOO_BAR1234")
        assert get_environment_var("${BAR_FOO1234}") == "BAR_FOO1234", get_environment_var("${BAR_FOO1234}")
        assert get_environment_var("${BAR}FOO") == None, get_environment_var("${BAR}FOO")
        assert get_environment_var("$BAR ") == None, get_environment_var("$BAR ")
        assert get_environment_var("FOO$BAR") == None, get_environment_var("FOO$BAR")
        assert get_environment_var("$FOO[0]") == None, get_environment_var("$FOO[0]")
        assert get_environment_var("${some('complex expression')}") == None, get_environment_var(
            "${some('complex expression')}")

    def test_Proxy(self):
        """Test generic Proxy class."""

        class Subject(object):
            def foo(self):
                return 1

            def bar(self):
                return 2

        s = Subject()
        s.baz = 3

        class ProxyTest(Proxy):
            def bar(self):
                return 4

        p = ProxyTest(s)

        assert p.foo() == 1, p.foo()
        assert p.bar() == 4, p.bar()
        assert p.baz == 3, p.baz

        p.baz = 5
        s.baz = 6

        assert p.baz == 5, p.baz
        assert p.get() == s, p.get()

    def test_display(self):
        old_stdout = sys.stdout
        sys.stdout = OutBuffer()
        display("line1")
        display.set_mode(0)
        display("line2")
        display.set_mode(1)
        display("line3")
        display("line4\n", append_newline=0)
        display.set_mode(0)
        display("dont print1")
        display("dont print2\n", append_newline=0)
        display.set_mode(1)
        assert sys.stdout.buffer == "line1\nline3\nline4\n"
        sys.stdout = old_stdout

    def test_get_native_path(self):
        """Test the get_native_path() function."""
        import tempfile
        filename = tempfile.mktemp()
        str = '1234567890 ' + filename
        try:
            with open(filename, 'w') as f:
                f.write(str)
            with open(get_native_path(filename)) as f:
                assert f.read() == str
        finally:
            try:
                os.unlink(filename)
            except OSError:
                pass

    def test_PrependPath(self):
        """Test prepending to a path"""
        p1 = r'C:\dir\num\one;C:\dir\num\two'
        p2 = r'C:\mydir\num\one;C:\mydir\num\two'
        # have to include the pathsep here so that the test will work on UNIX too.
        p1 = PrependPath(p1, r'C:\dir\num\two', sep=';')
        p1 = PrependPath(p1, r'C:\dir\num\three', sep=';')
        p2 = PrependPath(p2, r'C:\mydir\num\three', sep=';')
        p2 = PrependPath(p2, r'C:\mydir\num\one', sep=';')
        assert (p1 == r'C:\dir\num\three;C:\dir\num\two;C:\dir\num\one')
        assert (p2 == r'C:\mydir\num\one;C:\mydir\num\three;C:\mydir\num\two')

    def test_AppendPath(self):
        """Test appending to a path."""
        p1 = r'C:\dir\num\one;C:\dir\num\two'
        p2 = r'C:\mydir\num\one;C:\mydir\num\two'
        # have to include the pathsep here so that the test will work on UNIX too.
        p1 = AppendPath(p1, r'C:\dir\num\two', sep=';')
        p1 = AppendPath(p1, r'C:\dir\num\three', sep=';')
        p2 = AppendPath(p2, r'C:\mydir\num\three', sep=';')
        p2 = AppendPath(p2, r'C:\mydir\num\one', sep=';')
        assert (p1 == r'C:\dir\num\one;C:\dir\num\two;C:\dir\num\three')
        assert (p2 == r'C:\mydir\num\two;C:\mydir\num\three;C:\mydir\num\one')

    def test_PrependPathPreserveOld(self):
        """Test prepending to a path while preserving old paths"""
        p1 = r'C:\dir\num\one;C:\dir\num\two'
        # have to include the pathsep here so that the test will work on UNIX too.
        p1 = PrependPath(p1, r'C:\dir\num\two', sep=';', delete_existing=0)
        p1 = PrependPath(p1, r'C:\dir\num\three', sep=';')
        assert (p1 == r'C:\dir\num\three;C:\dir\num\one;C:\dir\num\two')

    def test_AppendPathPreserveOld(self):
        """Test appending to a path while preserving old paths"""
        p1 = r'C:\dir\num\one;C:\dir\num\two'
        # have to include the pathsep here so that the test will work on UNIX too.
        p1 = AppendPath(p1, r'C:\dir\num\one', sep=';', delete_existing=0)
        p1 = AppendPath(p1, r'C:\dir\num\three', sep=';')
        assert (p1 == r'C:\dir\num\one;C:\dir\num\two;C:\dir\num\three')

    def test_addPathIfNotExists(self):
        """Test the AddPathIfNotExists() function"""
        env_dict = {'FOO': os.path.normpath('/foo/bar') + os.pathsep + \
                           os.path.normpath('/baz/blat'),
                    'BAR': os.path.normpath('/foo/bar') + os.pathsep + \
                           os.path.normpath('/baz/blat'),
                    'BLAT': [os.path.normpath('/foo/bar'),
                             os.path.normpath('/baz/blat')]}
        AddPathIfNotExists(env_dict, 'FOO', os.path.normpath('/foo/bar'))
        AddPathIfNotExists(env_dict, 'BAR', os.path.normpath('/bar/foo'))
        AddPathIfNotExists(env_dict, 'BAZ', os.path.normpath('/foo/baz'))
        AddPathIfNotExists(env_dict, 'BLAT', os.path.normpath('/baz/blat'))
        AddPathIfNotExists(env_dict, 'BLAT', os.path.normpath('/baz/foo'))

        assert env_dict['FOO'] == os.path.normpath('/foo/bar') + os.pathsep + \
               os.path.normpath('/baz/blat'), env_dict['FOO']
        assert env_dict['BAR'] == os.path.normpath('/bar/foo') + os.pathsep + \
               os.path.normpath('/foo/bar') + os.pathsep + \
               os.path.normpath('/baz/blat'), env_dict['BAR']
        assert env_dict['BAZ'] == os.path.normpath('/foo/baz'), env_dict['BAZ']
        assert env_dict['BLAT'] == [os.path.normpath('/baz/foo'),
                                    os.path.normpath('/foo/bar'),
                                    os.path.normpath('/baz/blat')], env_dict['BLAT']

    def test_CLVar(self):
        """Test the command-line construction variable class"""
        f = SCons.Util.CLVar('a b')

        r = f + 'c d'
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', 'c', 'd'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + ' c d'
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', 'c', 'd'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + ['c d']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', 'c d'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + [' c d']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', ' c d'], r.data
        assert str(r) == 'a b  c d', str(r)

        r = f + ['c', 'd']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', 'c', 'd'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + [' c', 'd']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', ' c', 'd'], r.data
        assert str(r) == 'a b  c d', str(r)

        f = SCons.Util.CLVar(['a b'])

        r = f + 'c d'
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a b', 'c', 'd'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + ' c d'
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a b', 'c', 'd'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + ['c d']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a b', 'c d'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + [' c d']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a b', ' c d'], r.data
        assert str(r) == 'a b  c d', str(r)

        r = f + ['c', 'd']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a b', 'c', 'd'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + [' c', 'd']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a b', ' c', 'd'], r.data
        assert str(r) == 'a b  c d', str(r)

        f = SCons.Util.CLVar(['a', 'b'])

        r = f + 'c d'
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', 'c', 'd'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + ' c d'
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', 'c', 'd'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + ['c d']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', 'c d'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + [' c d']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', ' c d'], r.data
        assert str(r) == 'a b  c d', str(r)

        r = f + ['c', 'd']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', 'c', 'd'], r.data
        assert str(r) == 'a b c d', str(r)

        r = f + [' c', 'd']
        assert isinstance(r, SCons.Util.CLVar), type(r)
        assert r.data == ['a', 'b', ' c', 'd'], r.data
        assert str(r) == 'a b  c d', str(r)

    def test_Selector(self):
        """Test the Selector class"""

        class MyNode(object):
            def __init__(self, name):
                self.name = name

            def __str__(self):
                return self.name

            def get_suffix(self):
                return os.path.splitext(self.name)[1]

        s = Selector({'a': 'AAA', 'b': 'BBB'})
        assert s['a'] == 'AAA', s['a']
        assert s['b'] == 'BBB', s['b']
        exc_caught = None
        try:
            x = s['c']
        except KeyError:
            exc_caught = 1
        assert exc_caught, "should have caught a KeyError"
        s['c'] = 'CCC'
        assert s['c'] == 'CCC', s['c']

        class DummyEnv(UserDict):
            def subst(self, key):
                if key[0] == '$':
                    return self[key[1:]]
                return key

        env = DummyEnv()

        s = Selector({'.d': 'DDD', '.e': 'EEE'})
        ret = s(env, [])
        assert ret is None, ret
        ret = s(env, [MyNode('foo.d')])
        assert ret == 'DDD', ret
        ret = s(env, [MyNode('bar.e')])
        assert ret == 'EEE', ret
        ret = s(env, [MyNode('bar.x')])
        assert ret is None, ret
        s[None] = 'XXX'
        ret = s(env, [MyNode('bar.x')])
        assert ret == 'XXX', ret

        env = DummyEnv({'FSUFF': '.f', 'GSUFF': '.g'})

        s = Selector({'$FSUFF': 'FFF', '$GSUFF': 'GGG'})
        ret = s(env, [MyNode('foo.f')])
        assert ret == 'FFF', ret
        ret = s(env, [MyNode('bar.g')])
        assert ret == 'GGG', ret

    def test_adjustixes(self):
        """Test the adjustixes() function"""
        r = adjustixes('file', 'pre-', '-suf')
        assert r == 'pre-file-suf', r
        r = adjustixes('pre-file', 'pre-', '-suf')
        assert r == 'pre-file-suf', r
        r = adjustixes('file-suf', 'pre-', '-suf')
        assert r == 'pre-file-suf', r
        r = adjustixes('pre-file-suf', 'pre-', '-suf')
        assert r == 'pre-file-suf', r
        r = adjustixes('pre-file.xxx', 'pre-', '-suf')
        assert r == 'pre-file.xxx', r
        r = adjustixes('dir/file', 'pre-', '-suf')
        assert r == os.path.join('dir', 'pre-file-suf'), r

    def test_containsAny(self):
        """Test the containsAny() function"""
        assert containsAny('*.py', '*?[]')
        assert not containsAny('file.txt', '*?[]')

    def test_containsAll(self):
        """Test the containsAll() function"""
        assert containsAll('43221', '123')
        assert not containsAll('134', '123')

    def test_containsOnly(self):
        """Test the containsOnly() function"""
        assert containsOnly('.83', '0123456789.')
        assert not containsOnly('43221', '123')

    def test_LogicalLines(self):
        """Test the LogicalLines class"""
        content = u"""
foo \\
bar \\
baz
foo
bling \\
bling \\ bling
bling
"""
        fobj = io.StringIO(content)
        lines = LogicalLines(fobj).readlines()
        assert lines == [
            '\n',
            'foo bar baz\n',
            'foo\n',
            'bling bling \\ bling\n',
            'bling\n',
        ], lines

    def test_intern(self):
        s1 = silent_intern("spam")
        # TODO: Python 3.x does not have a unicode() global function
        if sys.version[0] == '2':
            s2 = silent_intern(unicode("unicode spam"))
        s3 = silent_intern(42)
        s4 = silent_intern("spam")
        assert id(s1) == id(s4)


class MD5TestCase(unittest.TestCase):

    def test_collect(self):
        """Test collecting a list of signatures into a new signature value
        """
        s = list(map(MD5signature, ('111', '222', '333')))

        assert '698d51a19d8a121ce581499d7b701668' == MD5collect(s[0:1])
        assert '8980c988edc2c78cc43ccb718c06efd5' == MD5collect(s[0:2])
        assert '53fd88c84ff8a285eb6e0a687e55b8c7' == MD5collect(s)

    def test_MD5signature(self):
        """Test generating a signature"""
        s = MD5signature('111')
        assert '698d51a19d8a121ce581499d7b701668' == s, s

        s = MD5signature('222')
        assert 'bcbe3365e6ac95ea2c0343a2395834dd' == s, s


class NodeListTestCase(unittest.TestCase):
    def test_simple_attributes(self):
        """Test simple attributes of a NodeList class"""

        class TestClass(object):
            def __init__(self, name, child=None):
                self.child = child
                self.bar = name

        t1 = TestClass('t1', TestClass('t1child'))
        t2 = TestClass('t2', TestClass('t2child'))
        t3 = TestClass('t3')

        nl = NodeList([t1, t2, t3])
        assert nl.bar == ['t1', 't2', 't3'], nl.bar
        assert nl[0:2].child.bar == ['t1child', 't2child'], \
            nl[0:2].child.bar

    def test_callable_attributes(self):
        """Test callable attributes of a NodeList class"""

        class TestClass(object):
            def __init__(self, name, child=None):
                self.child = child
                self.bar = name

            def foo(self):
                return self.bar + "foo"

            def getself(self):
                return self

        t1 = TestClass('t1', TestClass('t1child'))
        t2 = TestClass('t2', TestClass('t2child'))
        t3 = TestClass('t3')

        nl = NodeList([t1, t2, t3])
        assert nl.foo() == ['t1foo', 't2foo', 't3foo'], nl.foo()
        assert nl.bar == ['t1', 't2', 't3'], nl.bar
        assert nl.getself().bar == ['t1', 't2', 't3'], nl.getself().bar
        assert nl[0:2].child.foo() == ['t1childfoo', 't2childfoo'], \
            nl[0:2].child.foo()
        assert nl[0:2].child.bar == ['t1child', 't2child'], \
            nl[0:2].child.bar

    def test_null(self):
        """Test a null NodeList"""
        nl = NodeList([])
        r = str(nl)
        assert r == '', r
        for node in nl:
            raise Exception("should not enter this loop")


class flattenTestCase(unittest.TestCase):

    def test_scalar(self):
        """Test flattening a scalar"""
        result = flatten('xyz')
        self.assertEqual(result, ['xyz'], result)

    def test_dictionary_values(self):
        """Test flattening the dictionary values"""
        items = {"a": 1, "b": 2, "c": 3}
        result = flatten(items.values())
        self.assertEqual(sorted(result), [1, 2, 3])


class OsEnviron(object):
    """Used to temporarily mock os.environ"""

    def __init__(self, environ):
        self._environ = environ

    def start(self):
        self._stored = os.environ
        os.environ = self._environ

    def stop(self):
        os.environ = self._stored
        del self._stored

    def __enter__(self):
        self.start()
        return os.environ

    def __exit__(self, *args):
        self.stop()


class get_env_boolTestCase(unittest.TestCase):
    def test_missing(self):
        env = dict()
        var = get_env_bool(env, 'FOO')
        assert var is False, "var should be False, not %s" % repr(var)
        env = {'FOO': '1'}
        var = get_env_bool(env, 'BAR')
        assert var is False, "var should be False, not %s" % repr(var)

    def test_true(self):
        for foo in ['TRUE', 'True', 'true',
                    'YES', 'Yes', 'yes',
                    'Y', 'y',
                    'ON', 'On', 'on',
                    '1', '20', '-1']:
            env = {'FOO': foo}
            var = get_env_bool(env, 'FOO')
            assert var is True, 'var should be True, not %s' % repr(var)

    def test_false(self):
        for foo in ['FALSE', 'False', 'false',
                    'NO', 'No', 'no',
                    'N', 'n',
                    'OFF', 'Off', 'off',
                    '0']:
            env = {'FOO': foo}
            var = get_env_bool(env, 'FOO', True)
            assert var is False, 'var should be True, not %s' % repr(var)

    def test_default(self):
        env = {'FOO': 'other'}
        var = get_env_bool(env, 'FOO', True)
        assert var is True, 'var should be True, not %s' % repr(var)
        var = get_env_bool(env, 'FOO', False)
        assert var is False, 'var should be False, not %s' % repr(var)


class get_os_env_boolTestCase(unittest.TestCase):
    def test_missing(self):
        with OsEnviron(dict()):
            var = get_os_env_bool('FOO')
            assert var is False, "var should be False, not %s" % repr(var)
        with OsEnviron({'FOO': '1'}):
            var = get_os_env_bool('BAR')
            assert var is False, "var should be False, not %s" % repr(var)

    def test_true(self):
        for foo in ['TRUE', 'True', 'true',
                    'YES', 'Yes', 'yes',
                    'Y', 'y',
                    'ON', 'On', 'on',
                    '1', '20', '-1']:
            with OsEnviron({'FOO': foo}):
                var = get_os_env_bool('FOO')
                assert var is True, 'var should be True, not %s' % repr(var)

    def test_false(self):
        for foo in ['FALSE', 'False', 'false',
                    'NO', 'No', 'no',
                    'N', 'n',
                    'OFF', 'Off', 'off',
                    '0']:
            with OsEnviron({'FOO': foo}):
                var = get_os_env_bool('FOO', True)
                assert var is False, 'var should be True, not %s' % repr(var)

    def test_default(self):
        with OsEnviron({'FOO': 'other'}):
            var = get_os_env_bool('FOO', True)
            assert var is True, 'var should be True, not %s' % repr(var)
            var = get_os_env_bool('FOO', False)
            assert var is False, 'var should be False, not %s' % repr(var)


if __name__ == "__main__":
    unittest.main()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/scons-time/time/which.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify the time --which option.
"""

import TestSCons_time

test = TestSCons_time.TestSCons_time()

test.fake_logfile('foo-000-0.log', 0)
test.fake_logfile('foo-000-1.log', 0)
test.fake_logfile('foo-000-2.log', 0)

test.fake_logfile('foo-001-0.log', 1)
test.fake_logfile('foo-001-1.log', 1)
test.fake_logfile('foo-001-2.log', 1)

expect = """\
set key bottom left
plot '-' title "Startup" with lines lt 1, \\
     '-' title "Full build" with lines lt 2, \\
     '-' title "Up-to-date build" with lines lt 3
# Startup
0 %(time)s
1 %(time)s
e
# Full build
0 %(time)s
1 %(time)s
e
# Up-to-date build
0 %(time)s
1 %(time)s
e
"""

total       = expect % {'time' : 11.123456}
SConscripts = expect % {'time' : 22.234567}
SCons       = expect % {'time' : 33.345678}
commands    = expect % {'time' : 44.456789}

test.run(arguments = 'time --fmt gnuplot --which total', stdout=total)

test.run(arguments = 'time --fmt gnuplot --which=SConscripts', stdout=SConscripts)

test.run(arguments = 'time --fmt gnuplot --which=SCons', stdout=SCons)

test.run(arguments = 'time --fmt gnuplot --which commands', stdout=commands)

expect = """\
scons-time: time: Unrecognized timer "unknown".
            Type "scons-time help time" for help.
"""

test.run(arguments = 'time --fmt gnuplot --which unknown',
         status = 1,
         stderr = expect)

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
from __future__ import print_function

__revision__ = "test/TEX/TEX.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

r"""
Validate that we can set the TEX string to our own utility, that
the produced .dvi, .aux and .log files get removed by the -c option,
and that we can use this to wrap calls to the real latex utility.
"""

import TestSCons

_python_ = TestSCons._python_

test = TestSCons.TestSCons()

have_latex = test.where_is('latex')
if not have_latex:
    test.skip_test('Could not find latex; skipping test(s).\n')


test.write('mytex.py', r"""
import sys
import os
import getopt
cmd_opts, arg = getopt.getopt(sys.argv[1:], 'i:r:', [])
base_name = os.path.splitext(arg[0])[0]
with open(arg[0], 'r') as ifp:
    with open(base_name+'.dvi', 'w') as dvi_file, \
         open(base_name+'.aux', 'w') as aux_file, \
         open(base_name+'.log', 'w') as log_file:

        for l in ifp.readlines():
            if l[0] != '\\':
                dvi_file.write(l)
                aux_file.write(l)
                log_file.write(l)
sys.exit(0)
""")

test.write('SConstruct', """
env = Environment(TEX = r'%(_python_)s mytex.py', tools=['tex'])
env.DVI(target = 'test.dvi', source = 'test.tex')
""" % locals())

test.write('test.tex', r"""This is a test.
\end
""")

test.run(arguments = 'test.dvi')

test.must_match('test.dvi', "This is a test.\n")
test.must_match('test.aux', "This is a test.\n")
test.must_match('test.log', "This is a test.\n")

test.run(arguments = '-c test.dvi')

test.must_not_exist('test.dvi')
test.must_not_exist('test.aux')
test.must_not_exist('test.log')



tex = test.where_is('tex')

if tex:

    test.file_fixture('wrapper.py')

    test.write('SConstruct', """
import os
ENV = { 'PATH' : os.environ['PATH'] }
foo = Environment(ENV = ENV)
tex = foo.Dictionary('TEX')
bar = Environment(ENV = ENV, TEX = r'%(_python_)s wrapper.py ' + tex)
foo.DVI(target = 'foo.dvi', source = 'foo.tex')
foo.DVI(target = 'foo-latex.dvi', source = 'foo-latex.tex')
bar.DVI(target = 'bar', source = 'bar.tex')
bar.DVI(target = 'bar-latex', source = 'bar-latex.tex')
foo.DVI('rerun.tex')
foo.DVI('bibtex-test.tex')
""" % locals())

    tex = r"""
This is the %s TeX file.
\end
"""

    latex = r"""
\document%s{letter}
\begin{document}
This is the %s LaTeX file.
\end{document}
"""

    rerun = r"""
\documentclass{article}

\begin{document}

\LaTeX\ will need to run twice on this document to get a reference to section
\ref{sec}.

\section{Next section}
\label{sec}

\end{document}
"""

    bibtex = r"""
\documentclass{article}

\begin{document}

Run \texttt{latex}, then \texttt{bibtex}, then \texttt{latex} twice again \cite{lamport}.

\bibliographystyle{plain}
\bibliography{test}

\end{document}
"""

    bib = r"""
@Book{lamport,
  author =      {L. Lamport},
  title =       {{\LaTeX: A} Document Preparation System},
  publisher =   {Addison-Wesley},
  year =        1994
}
"""

    test.write('foo.tex', tex % 'foo.tex')
    test.write('bar.tex', tex % 'bar.tex')
    test.write('foo-latex.tex', latex % ('style', 'foo-latex.tex'))
    test.write('bar-latex.tex', latex % ('class', 'bar-latex.tex'))
    test.write('rerun.tex', rerun)
    test.write('bibtex-test.tex', bibtex)
    test.write('test.bib', bib)

    test.run(arguments = 'foo.dvi foo-latex.dvi', stderr = None)
    test.must_not_exist('wrapper.out')
    test.must_exist('foo.dvi')
    test.must_exist('foo-latex.dvi')

    test.run(arguments = 'bar.dvi bar-latex.dvi', stderr = None)
    test.must_exist('wrapper.out')
    test.must_exist('bar.dvi')
    test.must_exist('bar-latex.dvi')

    test.run(stderr = None)
    output_lines = test.stdout().split('\n')

    reruns = [x for x in output_lines if x.find('latex -interaction=nonstopmode -recorder rerun.tex') != -1]
    if len(reruns) != 2:
        print("Expected 2 latex calls, got %s:" % len(reruns))
        print('\n'.join(reruns))
        test.fail_test()

    bibtex = [x for x in output_lines if x.find('bibtex bibtex-test') != -1]
    if len(bibtex) != 1:
        print("Expected 1 bibtex call, got %s:" % len(bibtex))
        print('\n'.join(bibtex))
        test.fail_test()

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/LINK/LDMODULEVERSIONFLAGS.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import os
import re

import TestSCons
import SCons.Platform
import SCons.Defaults

foo_c_src = "void foo() {}\n"

env = SCons.Defaults.DefaultEnvironment()
platform = SCons.Platform.platform_default()
tool_list = SCons.Platform.DefaultToolList(platform, env)


test = TestSCons.TestSCons()
if 'gnulink' in tool_list:
    versionflags = r".+ -Wl,-Bsymbolic -Wl,-soname=libfoo.so.1( .+)+"
elif 'sunlink' in tool_list:
    versionflags = r".+ -h libfoo.so.1( .+)+"
else:
    test.skip_test('No testable linkers found, skipping the test\n')


# We expect stdout to not contain LDMODULEVERSIONFLAGS if there is no
# SHLIBVERSION nor LDMODULEVERSION provided
test.write('foo.c', foo_c_src)
test.write('SConstruct', "LoadableModule('foo','foo.c')\n")
test.run()
test.fail_test(test.match_re_dotall(test.stdout(), versionflags))
test.run(arguments = ['-c'])

for versionvar in ['SHLIBVERSION', 'LDMODULEVERSION']:
    test = TestSCons.TestSCons()
    test.write('foo.c', foo_c_src)
    test.write('SConstruct', "LoadableModule('foo','foo.c',%s='1.2.3')\n" % versionvar)
    test.run(stdout = versionflags, match = TestSCons.match_re_dotall)
    test.run(arguments = ['-c'])

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



import getopt
import sys
cmd_opts, args = getopt.getopt(sys.argv[1:], 'o:I:x', [])
opt_string = ''
i_arguments = ''
for opt, arg in cmd_opts:
    if opt == '-o': out = arg
    elif opt == '-I': i_arguments = i_arguments + ' ' + arg
    else: opt_string = opt_string + ' ' + opt
with open(out, 'wb') as ofp:
    for a in args:
        with open(a, 'rb') as ifp:
            contents = ifp.read()
        contents = contents.replace(b'YACCFLAGS', opt_string.encode())
        contents = contents.replace(b'I_ARGS', i_arguments.encode())
        ofp.write(contents)
sys.exit(0)



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/Fortran/FORTRANSUFFIXES.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Test the ability to scan additional filesuffixes added to $FORTRANSUFFIXES.
"""

import TestSCons

_python_ = TestSCons._python_

test = TestSCons.TestSCons()

test.write('myfc.py', r"""
import sys
def do_file(outf, inf):
    with open(inf, 'rb') as inf:
        for line in inf.readlines():
            if line[:15] == b"      INCLUDE '":
                do_file(outf, line[15:-2])
            else:
                outf.write(line)
with open(sys.argv[1], 'wb') as outf:
    for f in sys.argv[2:]:
        do_file(outf, f)
sys.exit(0)
""")

test.write('SConstruct', """
env = Environment(FORTRANPATH = ['.'],
                  FORTRAN = r'%(_python_)s myfc.py',
                  FORTRANCOM = '$FORTRAN $TARGET $SOURCES',
                  OBJSUFFIX = '.o')
env.Append(FORTRANSUFFIXES = ['.x'])
env.Object(target = 'test1', source = 'test1.f')
env.InstallAs('test1_f', 'test1.f')
env.InstallAs('test1_x', 'test1.x')
env.InstallAs('test2_f', 'test2.f')
""" % locals())

test.write('test1.f', """\
      test1.f 1
      INCLUDE 'test2.f'
      INCLUDE 'test1.x'
""")

test.write('test2.f', """\
      test2.f 1
      INCLUDE 'foo.f'
""")

test.write('test1.x', """\
      test1.x 1
      INCLUDE 'foo.f'
""")

test.write('foo.f', """\
      foo.f 1
""")

expect = test.wrap_stdout("""\
%(_python_)s myfc.py test1.o test1.f
Install file: "test1.f" as "test1_f"
Install file: "test1.x" as "test1_x"
Install file: "test2.f" as "test2_f"
""" % locals())

test.run(arguments='.', stdout=expect)

test.must_match('test1.o', """\
      test1.f 1
      test2.f 1
      foo.f 1
      test1.x 1
      foo.f 1
""")

test.up_to_date(arguments='.')

test.write('foo.f', """\
      foo.f 2
""")

expect = test.wrap_stdout("""\
%(_python_)s myfc.py test1.o test1.f
""" % locals())

test.run(arguments='.', stdout=expect)

test.must_match('test1.o', """\
      test1.f 1
      test2.f 1
      foo.f 2
      test1.x 1
      foo.f 2
""")

test.up_to_date(arguments='.')

test.write('test1.x', """\
      test1.x 2
      INCLUDE 'foo.f'
""")

expect = test.wrap_stdout("""\
%(_python_)s myfc.py test1.o test1.f
Install file: "test1.x" as "test1_x"
""" % locals())

test.run(arguments='.', stdout=expect)

test.must_match('test1.o', """\
      test1.f 1
      test2.f 1
      foo.f 2
      test1.x 2
      foo.f 2
""")

test.up_to_date(arguments='.')

test.write('test2.f', """\
      test2.f 2
      INCLUDE 'foo.f'
""")

expect = test.wrap_stdout("""\
%(_python_)s myfc.py test1.o test1.f
Install file: "test2.f" as "test2_f"
""" % locals())

test.run(arguments='.', stdout=expect)

test.must_match('test1.o', """\
      test1.f 1
      test2.f 2
      foo.f 2
      test1.x 2
      foo.f 2
""")

test.up_to_date(arguments='.')

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
__revision__ = "test/Interactive/shell.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify the ability of the "shell" command (and its "sh" and "!" aliases)
to shell out of interactive mode.
"""

import sys

import TestSCons

test = TestSCons.TestSCons(combine=1)

_python_ = TestSCons._python_

shell_command_py    = test.workpath('shell_command.py')
_shell_command_py_  = '"%s"' % shell_command_py.replace('\\', '\\\\')

test.write(shell_command_py, """\
from __future__ import print_function

print('hello from shell_command.py')
""")

test.write('SConstruct', """\
Command('foo.out', 'foo.in', Copy('$TARGET', '$SOURCE'))
Command('1', [], Touch('$TARGET'))
""")

test.write('foo.in', "foo.in 1\n")



scons = test.start(arguments = '-Q --interactive')

scons.send("build foo.out 1\n")

test.wait_for(test.workpath('1'))

test.must_match(test.workpath('foo.out'), "foo.in 1\n")



scons.send('!%(_python_)s %(_shell_command_py_)s\n' % locals())

scons.send("\n")

scons.send('shell %(_python_)s %(_shell_command_py_)s\n' % locals())

scons.send("\n")

scons.send('sh %(_python_)s %(_shell_command_py_)s\n' % locals())

scons.send("\n")

scons.send('!no_such_command arg1 arg2\n')

scons.send("\n")

scons.send("build foo.out\n")

scons.send("\n")

if sys.platform == 'win32':
    no_such_error = r"('no_such_command' is not recognized as an internal or external command,\s+operable program or batch file\.|Der Befehl \"no_such_command\" ist entweder falsch geschrieben oder\s+konnte nicht gefunden werden\.)"
else:
    no_such_error = 'scons: no_such_command: No such file or directory'

expect_stdout = \
r"""scons>>> Copy\("foo.out", "foo.in"\)
Touch\("1"\)
scons>>> hello from shell_command.py
scons>>> ![^"]+ ".*"
hello from shell_command.py
scons>>> hello from shell_command.py
scons>>> shell [^"]+ ".*"
hello from shell_command.py
scons>>> hello from shell_command.py
scons>>> sh [^"]+ ".*"
hello from shell_command.py
scons>>> %(no_such_error)s
scons>>> !no_such_command arg1 arg2
%(no_such_error)s
scons>>> scons: `foo.out' is up to date.
scons>>> build foo.out
scons: `foo.out' is up to date.
scons>>> 
""" % locals()

test.finish(scons, stdout = None)

test.must_contain_all_lines(test.stdout(), expect_stdout.splitlines(), find=TestSCons.search_re)

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/SideEffect/basic.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify basic operation of the SideEffect() method, using a "log
file" as the side effect "target."
"""

import os.path

import TestSCons

test = TestSCons.TestSCons()

test.write('SConstruct', """\
def copy(source, target):
    with open(target, "wb") as f, open(source, "rb") as f2:
        f.write(f2.read())

def build(env, source, target):
    copy(str(source[0]), str(target[0]))
    if target[0].side_effects:
        with open(str(target[0].side_effects[0]), "ab") as side_effect:
            side_effect.write(('%%s -> %%s\\n'%%(str(source[0]), str(target[0]))).encode())

Build = Builder(action=build)
env = Environment(BUILDERS={'Build':Build}, SUBDIR='subdir')
env.Build('foo.out', 'foo.in')
env.Build('bar.out', 'bar.in')
env.Build('blat.out', 'blat.in')
SideEffect('log.txt', ['foo.out', 'bar.out', 'blat.out'])
env.Build('log.out', 'log.txt')
env.Build('subdir/baz.out', 'baz.in')
env.SideEffect(r'%s', ['blat.out', r'%s'])
env.Build('subdir/out.out', 'subdir/out.txt')
""" % (os.path.join('$SUBDIR', 'out.txt'),
       os.path.join('$SUBDIR', 'baz.out')))

test.write('foo.in', 'foo.in\n')
test.write('bar.in', 'bar.in\n')
test.write('blat.in', 'blat.in\n')
test.write('baz.in', 'baz.in\n')

test.run(arguments = 'foo.out bar.out', stdout=test.wrap_stdout("""\
build(["foo.out"], ["foo.in"])
build(["bar.out"], ["bar.in"])
"""))

expect = """\
foo.in -> foo.out
bar.in -> bar.out
"""
test.must_match('log.txt', expect)

test.write('bar.in', 'bar.in 2 \n')

test.run(arguments = 'log.txt', stdout=test.wrap_stdout("""\
build(["bar.out"], ["bar.in"])
build(["blat.out"], ["blat.in"])
"""))

expect = expect + """\
bar.in -> bar.out
blat.in -> blat.out
"""
test.must_match('log.txt', expect)

test.write('foo.in', 'foo.in 2 \n')

test.run(arguments = ".", stdout=test.wrap_stdout("""\
build(["foo.out"], ["foo.in"])
build(["log.out"], ["log.txt"])
build(["%s"], ["baz.in"])
build(["%s"], ["%s"])
""" % (os.path.join('subdir', 'baz.out'),
       os.path.join('subdir', 'out.out'),
       os.path.join('subdir', 'out.txt'))))

expect = expect + """\
foo.in -> foo.out
"""

test.must_match('log.txt', expect)

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/Configure/option--config.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify use of the --config=<auto|force|cache> option.
"""

import os.path

import TestSCons

_obj = TestSCons._obj

test = TestSCons.TestSCons()

test.subdir('include')

NCR = test.NCR  # non-cached rebuild
CR = test.CR  # cached rebuild (up to date)
NCF = test.NCF  # non-cached build failure
CF = test.CF  # cached build failure

SConstruct_path = test.workpath('SConstruct')

test.write(SConstruct_path, """
env = Environment(CPPPATH='#/include')
import os
env.AppendENVPath('PATH', os.environ['PATH'])
conf = Configure(env)
conf.CheckCHeader('non_system_header0.h')
conf.CheckCHeader('non_system_header1.h')
env = conf.Finish()
""")

test.write(['include', 'non_system_header0.h'], """
/* A header */
""")

conftest_0_c = os.path.join(".sconf_temp", "conftest_0.c")
SConstruct_file_line = test.python_file_line(SConstruct_path, 6)[:-1]

expect = """
scons: *** "%(conftest_0_c)s" is not yet built and cache is forced.
%(SConstruct_file_line)s
""" % locals()

test.run(arguments='--config=cache', status=2, stderr=expect)

test.run(arguments='--config=auto')
test.checkLogAndStdout(["Checking for C header file non_system_header0.h... ",
                        "Checking for C header file non_system_header1.h... "],
                       ["yes", "no"],
                       [[((".c", NCR), (_obj, NCR))],
                        [((".c", NCR), (_obj, NCF))]],
                       "config.log", ".sconf_temp", "SConstruct")

test.run(arguments='--config=auto')
test.checkLogAndStdout(["Checking for C header file non_system_header0.h... ",
                        "Checking for C header file non_system_header1.h... "],
                       ["yes", "no"],
                       [[((".c", CR), (_obj, CR))],
                        [((".c", CR), (_obj, CF))]],
                       "config.log", ".sconf_temp", "SConstruct")

test.run(arguments='--config=force')
test.checkLogAndStdout(["Checking for C header file non_system_header0.h... ",
                        "Checking for C header file non_system_header1.h... "],
                       ["yes", "no"],
                       [[((".c", NCR), (_obj, NCR))],
                        [((".c", NCR), (_obj, NCF))]],
                       "config.log", ".sconf_temp", "SConstruct")

test.run(arguments='--config=cache')
test.checkLogAndStdout(["Checking for C header file non_system_header0.h... ",
                        "Checking for C header file non_system_header1.h... "],
                       ["yes", "no"],
                       [[((".c", CR), (_obj, CR))],
                        [((".c", CR), (_obj, CF))]],
                       "config.log", ".sconf_temp", "SConstruct")

test.write(['include', 'non_system_header1.h'], """
/* Another header */
""")
test.unlink(['include', 'non_system_header0.h'])

test.run(arguments='--config=cache')
test.checkLogAndStdout(["Checking for C header file non_system_header0.h... ",
                        "Checking for C header file non_system_header1.h... "],
                       ["yes", "no"],
                       [[((".c", CR), (_obj, CR))],
                        [((".c", CR), (_obj, CF))]],
                       "config.log", ".sconf_temp", "SConstruct")

test.run(arguments='--config=auto')
test.checkLogAndStdout(["Checking for C header file non_system_header0.h... ",
                        "Checking for C header file non_system_header1.h... "],
                       ["no", "yes"],
                       [[((".c", CR), (_obj, NCF))],
                        [((".c", CR), (_obj, NCR))]],
                       "config.log", ".sconf_temp", "SConstruct")


test.file_fixture('test_main.c')

# Check the combination of --config=force and Decider('MD5-timestamp')
SConstruct_path = test.workpath('SConstruct')
test.write(SConstruct_path, """
env = Environment()
env.Decider('MD5-timestamp')
conf = Configure(env)
conf.TryLink('int main(){return 0;}','.c')
env = conf.Finish()
env.Program('test_main.c')
""")
test.run(arguments='--config=force')
# On second run the sconsign is loaded and decider doesn't just indicate need to rebuild
test.run(arguments='--config=force')
test.must_not_contain(test.workpath('config.log'), "TypeError: 'NoneType' object is not callable", mode='r')

# Now check to check that test_main.c didn't rebuild on second run above.
# This fixes an issue where --config=force overwrites the Environments decider and is not reset when
# the configure context is done.
# https://github.com/SCons/scons/issues/3303
test.fail_test('test_main.o' in test.stdout())

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/Configure/VariantDir2.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Verify that Configure contexts work with SConstruct/SConscript structure
"""

import os

import TestSCons

test = TestSCons.TestSCons()

test.write('SConstruct', """\
SConscript('SConscript', variant_dir='build', src='.')
""")

test.write('SConscript', """\
env = Environment()
config = env.Configure(conf_dir='sconf', log_file='config.log')
config.TryRun("int main(void) {}", ".c")
config.Finish()
""")

test.run()
test.pass_test()



#!/usr/bin/env python
#
# Searches through the whole source tree and validates all
# documentation files against our own XSD in docs/xsd.
#
from __future__ import print_function

import sys,os
import SConsDoc

if __name__ == "__main__":
    if len(sys.argv)>1:
        if SConsDoc.validate_all_xml((sys.argv[1],)):
            print("OK")
        else:
            print("Validation failed! Please correct the errors above and try again.")
    else:
        if SConsDoc.validate_all_xml(['src',
                                      os.path.join('doc','design'),
                                      os.path.join('doc','developer'),
                                      os.path.join('doc','man'),
                                      os.path.join('doc','python10'),
                                      os.path.join('doc','reference'),
                                      os.path.join('doc','user')
                                      ]):
            print("OK")
        else:
            print("Validation failed! Please correct the errors above and try again.")
            sys.exit(1)



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/gettext/POUpdate/UserExamples.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Make sure, that the examples given in user guide all work.
"""

import TestSCons
import os

test = TestSCons.TestSCons()


if not test.where_is('msgmerge'):
    test.skip_test("Could not find 'msgmerge'; skipping test(s)\n")

pot_contents = """\
# SOME DESCRIPTIVE TITLE.
# Copyright (C) YEAR THE PACKAGE'S COPYRIGHT HOLDER
# This file is distributed under the same license as the PACKAGE package.
# FIRST AUTHOR <EMAIL@ADDRESS>, YEAR.
#
#, fuzzy
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"Report-Msgid-Bugs-To: \\n"
"POT-Creation-Date: 2012-05-27 00:35+0200\\n"
"PO-Revision-Date: YEAR-MO-DA HO:MI+ZONE\\n"
"Last-Translator: FULL NAME <EMAIL@ADDRESS>\\n"
"Language-Team: LANGUAGE <LL@li.org>\\n"
"Language: \\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=CHARSET\\n"
"Content-Transfer-Encoding: 8bit\\n"
#
#: a.cpp:1
msgid "Hello from a.cpp"
msgstr ""
"""

en_po_contents = """\
# English translations for PACKAGE package.
# Copyright (C) 2012 THE PACKAGE'S COPYRIGHT HOLDER
# This file is distributed under the same license as the PACKAGE package.
# Pawel Tomulik <ptomulik@meil.pw.edu.pl>, 2012.
#
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"Report-Msgid-Bugs-To: \\n"
"POT-Creation-Date: 2012-05-27 00:35+0200\\n"
"PO-Revision-Date: 2012-05-27 00:37+0200\\n"
"Last-Translator: Pawel Tomulik <ptomulik@meil.pw.edu.pl>\\n"
"Language-Team: English\\n"
"Language: en\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=ASCII\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Plural-Forms: nplurals=2; plural=(n != 1);\\n"

#: a.cpp:1
msgid "Old message from a.cpp"
msgstr "Old message from a.cpp"
"""

pl_po_contents = """\
# Polish translations for PACKAGE package.
# Copyright (C) 2012 THE PACKAGE'S COPYRIGHT HOLDER
# This file is distributed under the same license as the PACKAGE package.
# Automatically generated, 2012.
#
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"Report-Msgid-Bugs-To: \\n"
"POT-Creation-Date: 2012-05-27 00:35+0200\\n"
"PO-Revision-Date: 2012-05-27 00:35+0200\\n"
"Last-Translator: Automatically generated\\n"
"Language-Team: none\\n"
"Language: pl\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=ASCII\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Plural-Forms: nplurals=3; plural=(n==1 ? 0 : n%10>=2 && n%10<=4 && (n%100<10 "
"|| n%100>=20) ? 1 : 2);\\n"

#: a.cpp:1
msgid "Old message from a.cpp"
msgstr "Stara wiadomosc z a.cpp"
"""

de_po_contents = """\
# German translations for PACKAGE package.
# Copyright (C) 2012 THE PACKAGE'S COPYRIGHT HOLDER
# This file is distributed under the same license as the PACKAGE package.
# Automatically generated, 2012.
#
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"Report-Msgid-Bugs-To: \\n"
"POT-Creation-Date: 2012-05-27 00:35+0200\\n"
"PO-Revision-Date: 2012-05-27 00:35+0200\\n"
"Last-Translator: Automatically generated\\n"
"Language-Team: none\\n"
"Language: de\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=ASCII\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Plural-Forms: nplurals=2; plural=(n != 1);\\n"

#: a.cpp:1
msgid "Old message from a.cpp"
msgstr "EINE ALTE Nachricht vom a.cpp"
"""

fr_po_contents = """\
# French translations for PACKAGE package.
# Copyright (C) 2012 THE PACKAGE'S COPYRIGHT HOLDER
# This file is distributed under the same license as the PACKAGE package.
# Automatically generated, 2012.
#
msgid ""
msgstr ""
"Project-Id-Version: PACKAGE VERSION\\n"
"Report-Msgid-Bugs-To: \\n"
"POT-Creation-Date: 2012-05-27 00:35+0200\\n"
"PO-Revision-Date: 2012-05-27 00:35+0200\\n"
"Last-Translator: Automatically generated\\n"
"Language-Team: none\\n"
"Language: fr\\n"
"MIME-Version: 1.0\\n"
"Content-Type: text/plain; charset=ASCII\\n"
"Content-Transfer-Encoding: 8bit\\n"
"Plural-Forms: nplurals=2; plural=(n > 1);\\n"

#: a.cpp:1
msgid "Old message from a.cpp"
msgstr "Un ancien message du a.cpp"
"""

#############################################################################
# POUpdate: Example 1
#############################################################################
test.subdir(['ex1'])
test.write( ['ex1', 'SConstruct'],
"""
env = Environment( tools = ["default", "msgmerge"] )
env.POUpdate(['en','pl']) # messages.pot --&gt; [en.po, pl.po]
""")
test.write(['ex1', 'messages.pot'], pot_contents)
test.write(['ex1', 'en.po'], en_po_contents)
test.write(['ex1', 'pl.po'], pl_po_contents)

# NOTE: msgmerge(1) prints its messages to stderr, we must ignore them,
# So, stderr=None is crucial here. It is no point to match stderr to some
# specific valuse; the messages are internationalized :) ).
test.run(arguments = 'po-update', chdir = 'ex1', stderr = None)
test.must_exist(    ['ex1', 'en.po'] )
test.must_exist(    ['ex1', 'pl.po'] )
test.must_contain(  ['ex1', 'en.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex1', 'pl.po'], "Hello from a.cpp", mode='r' )

#############################################################################
# POUpdate: Example 2
#############################################################################
test.subdir(['ex2'])
test.write( ['ex2', 'SConstruct'],
"""
env = Environment( tools = ["default", "msgmerge"] )
env.POUpdate(['en','pl'], ['foo']) # foo.pot --&gt; [en.po, pl.po]
""")
#
test.write(['ex2', 'foo.pot'], pot_contents)
test.write(['ex2', 'en.po'], en_po_contents)
test.write(['ex2', 'pl.po'], pl_po_contents)

# NOTE: msgmerge(1) prints all messages to stderr, we must ignore them,
# So, stderr=None is crucial here. It is no point to match stderr to some
# specific valuse; the messages are internationalized :) ).
test.run(arguments = 'po-update', chdir = 'ex2', stderr = None)
test.must_exist(    ['ex2', 'en.po'] )
test.must_exist(    ['ex2', 'pl.po'] )
test.must_contain(  ['ex2', 'en.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex2', 'pl.po'], "Hello from a.cpp", mode='r' )

#############################################################################
# POUpdate: Example 3
#############################################################################
test.subdir(['ex3'])
test.write( ['ex3', 'SConstruct'],
"""
env = Environment( tools = ["default", "msgmerge"] )
env.POUpdate(['en','pl'], POTDOMAIN='foo') # foo.pot --&gt; [en.po, pl.po]
""")
#
test.write(['ex3', 'foo.pot'], pot_contents)
test.write(['ex3', 'en.po'], en_po_contents)
test.write(['ex3', 'pl.po'], pl_po_contents)

# NOTE: msgmerge(1) prints its messages to stderr, we must ignore them,
# So, stderr=None is crucial here. It is no point to match stderr to some
# specific valuse; the messages are internationalized :) ).
test.run(arguments = 'po-update', chdir = 'ex3', stderr = None)
test.must_exist(    ['ex3', 'en.po'] )
test.must_exist(    ['ex3', 'pl.po'] )
test.must_contain(  ['ex3', 'en.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex3', 'pl.po'], "Hello from a.cpp", mode='r' )

#############################################################################
# POUpdate: Example 4
#############################################################################
test.subdir(['ex4'])
test.write( ['ex4', 'SConstruct'],
"""
env = Environment( tools = ["default", "msgmerge"] )
env.POUpdate(LINGUAS_FILE = 1) # needs 'LINGUAS' file
""")
#
test.write(['ex4', 'LINGUAS'],
"""
en
pl
""")
#
test.write(['ex4', 'messages.pot'], pot_contents)
test.write(['ex4', 'en.po'], en_po_contents)
test.write(['ex4', 'pl.po'], pl_po_contents)

# NOTE: msgmerge(1) prints all messages to stderr, we must ignore them,
# So, stderr=None is crucial here. It is no point to match stderr to some
# specific valuse; the messages are internationalized :) ).
test.run(arguments = 'po-update', chdir = 'ex4', stderr = None)
test.must_exist(    ['ex4', 'messages.pot'] )
test.must_exist(    ['ex4', 'en.po'] )
test.must_exist(    ['ex4', 'pl.po'] )
test.must_contain(  ['ex4', 'en.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex4', 'pl.po'], "Hello from a.cpp", mode='r' )

#############################################################################
# POUpdate: Example 5
#############################################################################
test.subdir(['ex5'])
test.write( ['ex5', 'SConstruct'],
"""
env = Environment( tools = ["default", "msgmerge"] )
env.POUpdate(LINGUAS_FILE = 1, source = ['foo']) 
""")
test.write(['ex5', 'LINGUAS'],
"""
en
pl
""")
#
test.write(['ex5', 'foo.pot'], pot_contents)
test.write(['ex5', 'en.po'], en_po_contents)
test.write(['ex5', 'pl.po'], pl_po_contents)

# NOTE: msgmerge(1) prints its messages to stderr, we must ignore them,
# So, stderr=None is crucial here. It is no point to match stderr to some
# specific valuse; the messages are internationalized :) ).
test.run(arguments = 'po-update', chdir=  'ex5', stderr = None)
test.must_exist(    ['ex5', 'en.po'] )
test.must_exist(    ['ex5', 'pl.po'] )
test.must_contain(  ['ex5', 'en.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex5', 'pl.po'], "Hello from a.cpp", mode='r' )

#############################################################################
# POUpdate: Example 6
#############################################################################
test.subdir(['ex6'])
test.write( ['ex6', 'SConstruct'],
"""
env = Environment( tools = ["default", "msgmerge"] )
env.POUpdate(['en', 'pl'], LINGUAS_FILE = 1)
""")
test.write(['ex6', 'LINGUAS'],
"""
de
fr
""")
test.write(['ex6', 'messages.pot'], pot_contents)
test.write(['ex6', 'en.po'], en_po_contents)
test.write(['ex6', 'pl.po'], pl_po_contents)
test.write(['ex6', 'de.po'], de_po_contents)
test.write(['ex6', 'fr.po'], fr_po_contents)

# Note: msgmerge(1) prints its messages to stderr, we must ignore them,
# So, stderr=None is crucial here. It is no point to match stderr to some
# specific valuse; the messages are internationalized :) ).
test.run(arguments = 'po-update', chdir = 'ex6', stderr = None)
test.must_exist(    ['ex6', 'en.po'] )
test.must_exist(    ['ex6', 'pl.po'] )
test.must_exist(    ['ex6', 'de.po'] )
test.must_exist(    ['ex6', 'fr.po'] )
test.must_contain(  ['ex6', 'en.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex6', 'pl.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex6', 'de.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex6', 'fr.po'], "Hello from a.cpp", mode='r' )

#############################################################################
# POUpdate: Example 7
#############################################################################
#
# From this poin we need msginit
#
if not test.where_is('msginit'):
    test.skip_test("could not find 'msginit'; skipping test(s)\n")
###
test.subdir(['ex7'])
test.write( ['ex7', 'SConstruct'],
"""
env = Environment( tools = ["default", "msginit", "msgmerge"] )
env.POUpdate(LINGUAS_FILE = 1, POAUTOINIT = 1) 
""")
test.write(['ex7', 'LINGUAS'],
"""
en
pl
""")
#
test.write(['ex7', 'messages.pot'], pot_contents)

# NOTE: msgmerge(1) prints its messages to stderr, we must ignore them,
# So, stderr=None is crucial here. It is no point to match stderr to some
# specific valuse; the messages are internationalized :) ).
test.run(arguments = 'po-update', chdir=  'ex7', stderr = None)
test.must_exist(    ['ex7', 'en.po'] )
test.must_exist(    ['ex7', 'pl.po'] )
test.must_contain(  ['ex7', 'en.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex7', 'pl.po'], "Hello from a.cpp", mode='r' )

#############################################################################
# POUpdate: Example 8
#############################################################################
#
# From this point we need xgettext
#
if not test.where_is('xgettext'):
    test.skip_test("could not find 'xgettext'; skipping test(s)\n")
###
test.subdir(['ex8'])
test.write( ['ex8', 'SConstruct'],
"""
env = Environment( tools = ["default", "xgettext", "msginit", "msgmerge"] )

# script-wise settings
env['POAUTOINIT'] = 1
env['LINGUAS_FILE'] = 1
env['POTDOMAIN'] = 'foo'
env.POTUpdate(source = 'a.cpp')
env.POUpdate()
""")
test.write(['ex8', 'LINGUAS'],
"""
en
pl
""")
test.write(['ex8', 'a.cpp'], """ gettext("Hello from a.cpp") """)

# Note: msgmerge(1) prints its messages to stderr, we must ignore them,
# So, stderr=None is crucial here. It is no point to match stderr to some
# specific valuse; the messages are internationalized :) ).
test.run(arguments = 'po-update', chdir = 'ex8', stderr = None)
test.must_exist(    ['ex8', 'foo.pot'] )
test.must_exist(    ['ex8', 'en.po'] )
test.must_exist(    ['ex8', 'pl.po'] )
test.must_contain(  ['ex8', 'en.po'], "Hello from a.cpp", mode='r' )
test.must_contain(  ['ex8', 'pl.po'], "Hello from a.cpp", mode='r' )

test.pass_test()


# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/option/taskmastertrace.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

"""
Simple tests of the --taskmastertrace= option.
"""

import TestSCons

test = TestSCons.TestSCons()

test.write('SConstruct', """
DefaultEnvironment(tools=[])
env = Environment(tools=[])

# We name the files 'Tfile' so that they will sort after the SConstruct
# file regardless of whether the test is being run on a case-sensitive
# or case-insensitive system.

env.Command('Tfile.out', 'Tfile.mid', Copy('$TARGET', '$SOURCE'))
env.Command('Tfile.mid', 'Tfile.in', Copy('$TARGET', '$SOURCE'))
""")

test.write('Tfile.in', "Tfile.in\n")

expect_stdout = test.wrap_stdout("""\

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <no_state   0   '.'> and its children:
Taskmaster:        <no_state   0   'SConstruct'>
Taskmaster:        <no_state   0   'Tfile.in'>
Taskmaster:        <no_state   0   'Tfile.mid'>
Taskmaster:        <no_state   0   'Tfile.out'>
Taskmaster:      adjusted ref count: <pending    1   '.'>, child 'SConstruct'
Taskmaster:      adjusted ref count: <pending    2   '.'>, child 'Tfile.in'
Taskmaster:      adjusted ref count: <pending    3   '.'>, child 'Tfile.mid'
Taskmaster:      adjusted ref count: <pending    4   '.'>, child 'Tfile.out'
Taskmaster:     Considering node <no_state   0   'SConstruct'> and its children:
Taskmaster: Evaluating <pending    0   'SConstruct'>

Task.make_ready_current(): node <pending    0   'SConstruct'>
Task.prepare():      node <up_to_date 0   'SConstruct'>
Task.executed_with_callbacks(): node <up_to_date 0   'SConstruct'>
Task.postprocess():  node <up_to_date 0   'SConstruct'>
Task.postprocess():  removing <up_to_date 0   'SConstruct'>
Task.postprocess():  adjusted parent ref count <pending    3   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <no_state   0   'Tfile.in'> and its children:
Taskmaster: Evaluating <pending    0   'Tfile.in'>

Task.make_ready_current(): node <pending    0   'Tfile.in'>
Task.prepare():      node <up_to_date 0   'Tfile.in'>
Task.executed_with_callbacks(): node <up_to_date 0   'Tfile.in'>
Task.postprocess():  node <up_to_date 0   'Tfile.in'>
Task.postprocess():  removing <up_to_date 0   'Tfile.in'>
Task.postprocess():  adjusted parent ref count <pending    2   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <no_state   0   'Tfile.mid'> and its children:
Taskmaster:        <up_to_date 0   'Tfile.in'>
Taskmaster: Evaluating <pending    0   'Tfile.mid'>

Task.make_ready_current(): node <pending    0   'Tfile.mid'>
Task.prepare():      node <executing  0   'Tfile.mid'>
Task.execute():      node <executing  0   'Tfile.mid'>
Copy("Tfile.mid", "Tfile.in")
Task.executed_with_callbacks(): node <executing  0   'Tfile.mid'>
Task.postprocess():  node <executed   0   'Tfile.mid'>
Task.postprocess():  removing <executed   0   'Tfile.mid'>
Task.postprocess():  adjusted parent ref count <pending    1   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <no_state   0   'Tfile.out'> and its children:
Taskmaster:        <executed   0   'Tfile.mid'>
Taskmaster: Evaluating <pending    0   'Tfile.out'>

Task.make_ready_current(): node <pending    0   'Tfile.out'>
Task.prepare():      node <executing  0   'Tfile.out'>
Task.execute():      node <executing  0   'Tfile.out'>
Copy("Tfile.out", "Tfile.mid")
Task.executed_with_callbacks(): node <executing  0   'Tfile.out'>
Task.postprocess():  node <executed   0   'Tfile.out'>
Task.postprocess():  removing <executed   0   'Tfile.out'>
Task.postprocess():  adjusted parent ref count <pending    0   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <pending    0   '.'> and its children:
Taskmaster:        <up_to_date 0   'SConstruct'>
Taskmaster:        <up_to_date 0   'Tfile.in'>
Taskmaster:        <executed   0   'Tfile.mid'>
Taskmaster:        <executed   0   'Tfile.out'>
Taskmaster: Evaluating <pending    0   '.'>

Task.make_ready_current(): node <pending    0   '.'>
Task.prepare():      node <executing  0   '.'>
Task.execute():      node <executing  0   '.'>
Task.executed_with_callbacks(): node <executing  0   '.'>
Task.postprocess():  node <executed   0   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster: No candidate anymore.

""")

test.run(arguments='--taskmastertrace=- .', stdout=expect_stdout)



test.run(arguments='-c .')



expect_stdout = test.wrap_stdout("""\
Copy("Tfile.mid", "Tfile.in")
Copy("Tfile.out", "Tfile.mid")
""")

test.run(arguments='--taskmastertrace=trace.out .', stdout=expect_stdout)

expect_trace = """\

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <no_state   0   '.'> and its children:
Taskmaster:        <no_state   0   'SConstruct'>
Taskmaster:        <no_state   0   'Tfile.in'>
Taskmaster:        <no_state   0   'Tfile.mid'>
Taskmaster:        <no_state   0   'Tfile.out'>
Taskmaster:      adjusted ref count: <pending    1   '.'>, child 'SConstruct'
Taskmaster:      adjusted ref count: <pending    2   '.'>, child 'Tfile.in'
Taskmaster:      adjusted ref count: <pending    3   '.'>, child 'Tfile.mid'
Taskmaster:      adjusted ref count: <pending    4   '.'>, child 'Tfile.out'
Taskmaster:     Considering node <no_state   0   'SConstruct'> and its children:
Taskmaster: Evaluating <pending    0   'SConstruct'>

Task.make_ready_current(): node <pending    0   'SConstruct'>
Task.prepare():      node <up_to_date 0   'SConstruct'>
Task.executed_with_callbacks(): node <up_to_date 0   'SConstruct'>
Task.postprocess():  node <up_to_date 0   'SConstruct'>
Task.postprocess():  removing <up_to_date 0   'SConstruct'>
Task.postprocess():  adjusted parent ref count <pending    3   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <no_state   0   'Tfile.in'> and its children:
Taskmaster: Evaluating <pending    0   'Tfile.in'>

Task.make_ready_current(): node <pending    0   'Tfile.in'>
Task.prepare():      node <up_to_date 0   'Tfile.in'>
Task.executed_with_callbacks(): node <up_to_date 0   'Tfile.in'>
Task.postprocess():  node <up_to_date 0   'Tfile.in'>
Task.postprocess():  removing <up_to_date 0   'Tfile.in'>
Task.postprocess():  adjusted parent ref count <pending    2   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <no_state   0   'Tfile.mid'> and its children:
Taskmaster:        <up_to_date 0   'Tfile.in'>
Taskmaster: Evaluating <pending    0   'Tfile.mid'>

Task.make_ready_current(): node <pending    0   'Tfile.mid'>
Task.prepare():      node <executing  0   'Tfile.mid'>
Task.execute():      node <executing  0   'Tfile.mid'>
Task.executed_with_callbacks(): node <executing  0   'Tfile.mid'>
Task.postprocess():  node <executed   0   'Tfile.mid'>
Task.postprocess():  removing <executed   0   'Tfile.mid'>
Task.postprocess():  adjusted parent ref count <pending    1   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <no_state   0   'Tfile.out'> and its children:
Taskmaster:        <executed   0   'Tfile.mid'>
Taskmaster: Evaluating <pending    0   'Tfile.out'>

Task.make_ready_current(): node <pending    0   'Tfile.out'>
Task.prepare():      node <executing  0   'Tfile.out'>
Task.execute():      node <executing  0   'Tfile.out'>
Task.executed_with_callbacks(): node <executing  0   'Tfile.out'>
Task.postprocess():  node <executed   0   'Tfile.out'>
Task.postprocess():  removing <executed   0   'Tfile.out'>
Task.postprocess():  adjusted parent ref count <pending    0   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster:     Considering node <pending    0   '.'> and its children:
Taskmaster:        <up_to_date 0   'SConstruct'>
Taskmaster:        <up_to_date 0   'Tfile.in'>
Taskmaster:        <executed   0   'Tfile.mid'>
Taskmaster:        <executed   0   'Tfile.out'>
Taskmaster: Evaluating <pending    0   '.'>

Task.make_ready_current(): node <pending    0   '.'>
Task.prepare():      node <executing  0   '.'>
Task.execute():      node <executing  0   '.'>
Task.executed_with_callbacks(): node <executing  0   '.'>
Task.postprocess():  node <executed   0   '.'>

Taskmaster: Looking for a node to evaluate
Taskmaster: No candidate anymore.

"""

test.must_match('trace.out', expect_trace, mode='r')

test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:



#!/usr/bin/env python
#
# Copyright (c) 2001 - 2019 The SCons Foundation
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

__revision__ = "test/Alias/Depends.py bee7caf9defd6e108fc2998a2520ddb36a967691 2019-12-17 02:07:09 bdeegan"

import os

import TestSCons
import TestCmd

_python_ = TestSCons._python_

test = TestSCons.TestSCons(match=TestCmd.match_re)

test.subdir('sub1', 'sub2')

test.write('build.py', r"""
import sys
with open(sys.argv[1], 'wb') as f, open(sys.argv[2], 'rb') as ifp:
    f.write(ifp.read())
sys.exit(0)
""")

test.write('SConstruct', """
B = Builder(action = r'%(_python_)s build.py $TARGET $SOURCES')
env = Environment()
env['BUILDERS']['B'] = B
env.B(target = 'f1.out', source = 'f1.in')
env.B(target = 'f2.out', source = 'f2.in')
env.B(target = 'f3.out', source = 'f3.in')
SConscript('sub1/SConscript', "env")
SConscript('sub2/SConscript', "env")

foo = Alias('foo')
foo2 = env.Alias('foo', ['f2.out', 'sub1'])
assert foo == foo2
bar = Alias('bar', ['sub2', 'f3.out'])
env.Alias('blat', ['sub2', 'f3.out'])
env.Alias('blat', ['f2.out', 'sub1'])
env.Depends('f1.out', 'bar')

Alias('a1', 'a1-file.in')
Depends(Alias('a2'), 'a1')
env.B('a2-file.out', 'a2-file.in')
Depends('a2-file.out', 'a2')
""" % locals())

test.write(['sub1', 'SConscript'], """
Import("env")
env.B(target = 'f4.out', source = 'f4.in')
env.B(target = 'f5.out', source = 'f5.in')
env.B(target = 'f6.out', source = 'f6.in')
""")

test.write(['sub2', 'SConscript'], """
Import("env")
env.B(target = 'f7.out', source = 'f7.in')
env.B(target = 'f8.out', source = 'f8.in')
env.B(target = 'f9.out', source = 'f9.in')
""")

test.write('f1.in', "f1.in\n")
test.write('f2.in', "f2.in\n")
test.write('f3.in', "f3.in\n")

test.write(['sub1', 'f4.in'], "sub1/f4.in\n")
test.write(['sub1', 'f5.in'], "sub1/f5.in\n")
test.write(['sub1', 'f6.in'], "sub1/f6.in\n")

test.write(['sub2', 'f7.in'], "sub2/f7.in\n")
test.write(['sub2', 'f8.in'], "sub2/f8.in\n")
test.write(['sub2', 'f9.in'], "sub2/f9.in\n")



test.run(arguments = 'foo')

test.must_not_exist(test.workpath('f1.out'))
test.must_exist(test.workpath('f2.out'))
test.must_not_exist(test.workpath('f3.out'))

test.must_exist(test.workpath('sub1', 'f4.out'))
test.must_exist(test.workpath('sub1', 'f5.out'))
test.must_exist(test.workpath('sub1', 'f6.out'))

test.must_not_exist(test.workpath('sub2', 'f7.out'))
test.must_not_exist(test.workpath('sub2', 'f8.out'))
test.must_not_exist(test.workpath('sub2', 'f9.out'))

test.up_to_date(arguments = 'foo')

test.write(['sub1', 'f5.in'], "sub1/f5.in 2\n")

test.not_up_to_date(arguments = 'foo')



test.run(arguments = 'f1.out')

test.must_exist(test.workpath('f1.out'))
test.must_exist(test.workpath('f3.out'))

test.up_to_date(arguments = 'f1.out')



os.unlink(test.workpath('f2.out'))
os.unlink(test.workpath('f3.out'))

test.run(arguments = 'blat')

test.must_exist(test.workpath('f2.out'))
test.must_exist(test.workpath('f3.out'))

test.write('f3.in', "f3.in 2 \n")

expect = """.* build.py f3.out f3.in
.* build.py f1.out f1.in
"""

test.run(arguments = '-Q f1.out', stdout = expect)

test.up_to_date(arguments = 'f1.out')



test.write('a1-file.in', "a1-file.in\n")
test.write('a2-file.in', "a2-file.in\n")

test.run(arguments = 'a2-file.out')

test.must_match(test.workpath('a2-file.out'), "a2-file.in\n")

test.up_to_date(arguments = 'a2-file.out')

test.write('a1-file.in', "a1-file.in 2\n")

test.not_up_to_date(arguments = 'a2-file.out')



test.pass_test()

# Local Variables:
# tab-width:4
# indent-tabs-mode:nil
# End:
# vim: set expandtab tabstop=4 shiftwidth=4:
